
# -*- coding: us-ascii -*-
# vim:ts=4:sw=4:softtabstop=4:smarttab:expandtab
#
#    Copyright (C) 2009 Keith Dart <keith@dartworks.biz>
#
#    This library is free software; you can redistribute it and/or
#    modify it under the terms of the GNU Lesser General Public
#    License as published by the Free Software Foundation; either
#    version 2.1 of the License, or (at your option) any later version.
#
#    This library is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#    Lesser General Public License for more details.

"""
Collect system statistics and make them available on a Dbus channel.

"""

import sys
import os
import syslog
#import getopt

import gobject
import dbus
import dbus.service
import dbus.mainloop.glib

from pycopia.OS import procfs
from pycopia import scheduler


class PycopiaSystemServerError(dbus.DBusException):
    _dbus_error_name = 'net.pycopia.PycopiaSystemServerError'


class SystemStatServer(dbus.service.Object):

    @dbus.service.method("net.pycopia.SystemStatServer", in_signature='', out_signature='s')
    def Ping(self):
        return "Pong"

# XXX


def run():
    global mainloop
    dbus.mainloop.glib.DBusGMainLoop(set_as_default=True)
    system_bus = dbus.SystemBus()
    name = dbus.service.BusName("net.pycopia.SystemStatServer", system_bus)
    srv = HytestServer(system_bus, '/SystemStatServer')
    mainloop = gobject.MainLoop()
    syslog.openlog("systemstatserver")
    try:
        mainloop.run()
    finally:
        syslog.closelog()
        mainloop = None


def systemstatd(argv):
    do_daemon = True
    killserver = False
    try:
        optlist, args = getopt.getopt(argv[1:], "Dnh?kl:p:")
    except getopt.GetoptError:
        print systemstatd._doc % (argv[0],)
        return
    servername = os.path.basename(argv[0])
    logfilename = "/var/log/%s.log" % (servername,)
    pidfile="/var/run/%s.pid" % (servername,)

    for opt, optarg in optlist:
        if opt == "-n":
            do_daemon = False
        elif opt == "-k":
            killserver = True
        elif opt == "-D":
            from pycopia import autodebug
            do_daemon = False
        elif opt == "-l":
            logfilename = optarg
        elif opt == "-p":
            pidfile = optarg
        elif opt in ("-h", "-?"):
            print systemstatd._doc % (argv[0],)
            return 2

    if killserver:
        import signal
        if os.path.exists(pidfile):
            pid = int(open(pidfile).read().strip())
            os.unlink(pidfile)
            s = procfs.ProcStat(pid)
            if s:
                if s.command.find(servername) >= 0:
                    os.kill(pid, signal.SIGTERM)
                    print "Killed %s (%s)." % (servername, pid)
                else:
                    print "%s not us, it's %s." % (pid, s.command)
            else:
                print "%s not running." % (servername,)
                return 2
        return 0

    if do_daemon:
        from pycopia import daemonize
        from pycopia import logfile
        lf = logfile.ManagedStdio(logfilename)
        daemonize.daemonize(lf, pidfile=pidfile)
    else:
        fo = open(pidfile, "w")
        fo.write("%s\n" % (os.getpid(),))
        fo.close()
        del fo

    run()


systemstatd._doc = """Run the Pycopia system stats server.

    Provides data on dbus location net.pycopia.SystemStatServer

    %s [-ndk?] [-l <logfile>] [-p <pidfile>]

    Options:
         -n = Do NOT become a deamon.
         -D = Enable debugging. Also does not become a deamon.
         -k = Kill a running server.
         -l <filename> = Path name of file to log output to.
         -p <pidfile> = Path to PID file.
    """


if __name__ == "__main__":
    systemstatd(sys.argv)

